"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const lruCache = require("lru-cache");
const inceptum_1 = require("inceptum");
const AggregateEventStore_1 = require("./event/store/AggregateEventStore");
const ExecutionContext_1 = require("./ExecutionContext");
const Aggregate_1 = require("./Aggregate");
const Command_1 = require("./command/Command");
const EventExecutor_1 = require("./event/EventExecutor");
const MAX_AGGREGATE_CACHE_ENTRIES = 1000;
const MAX_AGGREGATE_CACHE_AGE = 1000 * 60 * 60; // one hour
const Logger = inceptum_1.LogManager.getLogger(__filename);
class CacheInvalidatingAggregateEventStore extends AggregateEventStore_1.AggregateEventStore {
    constructor(baseEventStore, cqrs) {
        super();
        this.baseEventStore = baseEventStore;
        this.cqrs = cqrs;
    }
    getEventsOf(aggregateId) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.baseEventStore.getEventsOf(aggregateId);
        });
    }
    commitEvent(aggregateEvent) {
        return __awaiter(this, void 0, void 0, function* () {
            const executor = EventExecutor_1.EventExecutor.getEventExecutor(aggregateEvent, this.cqrs.eventExecutors);
            if (!executor) {
                throw new Error(`Unknown event to commit ${aggregateEvent.constructor.name}`);
            }
            this.cqrs.aggregateCache.del(executor.getAggregateId(aggregateEvent));
            yield this.baseEventStore.commitEvent(aggregateEvent);
            this.cqrs.eventStream.emit('event', aggregateEvent);
            yield this.notifyEventListeners(aggregateEvent);
        });
    }
    commitAllEvents(aggregateEvents) {
        return __awaiter(this, void 0, void 0, function* () {
            aggregateEvents.forEach((aggregateEvent) => {
                const executor = EventExecutor_1.EventExecutor.getEventExecutor(aggregateEvent, this.cqrs.eventExecutors);
                if (!executor) {
                    throw new Error(`Unknown event to commit ${aggregateEvent.constructor.name}`);
                }
                this.cqrs.aggregateCache.del(executor.getAggregateId(aggregateEvent));
            });
            yield this.baseEventStore.commitAllEvents(aggregateEvents);
            aggregateEvents.forEach((aggregateEvent) => this.cqrs.eventStream.emit('event', aggregateEvent));
            yield aggregateEvents.reduce((prevPromise, aggregateEvent) => __awaiter(this, void 0, void 0, function* () {
                yield prevPromise;
                yield this.notifyEventListeners(aggregateEvent);
            }), Promise.resolve());
        });
    }
    notifyEventListeners(aggregateEvent) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.cqrs.eventListeners.reduce((prevPromise, eventListener) => __awaiter(this, void 0, void 0, function* () {
                yield prevPromise;
                try {
                    yield eventListener.eventCommitted(aggregateEvent);
                }
                catch (e) {
                    Logger.error(e, 'There was an error in event listener');
                }
            }), Promise.resolve());
        });
    }
}
class EventListener {
}
exports.EventListener = EventListener;
class CQRS {
    /**
     * Construct a new instance of the CQRS framework
     * @param {AggregateEventStore} aggregateEventStore The event store to use.
     */
    constructor(aggregateEventStore) {
        this.commandClasses = new Map();
        this.aggregateClasses = new Map();
        this.eventStream = new stream_1.Stream();
        this.commandExecutors = [];
        this.eventExecutors = [];
        this.eventListeners = [];
        this.aggregateCache = lruCache({
            max: MAX_AGGREGATE_CACHE_ENTRIES,
            maxAge: MAX_AGGREGATE_CACHE_AGE,
        });
        this.aggregateEventStore = new CacheInvalidatingAggregateEventStore(aggregateEventStore, this);
    }
    doSetup() {
        if (this.aggregateDefinitionsToRegister && this.aggregateDefinitionsToRegister.length > 0) {
            this.aggregateDefinitionsToRegister.forEach((def) => {
                const name = def.getProducedClass().aggregateName;
                if (!name) {
                    throw new Error(`Class ${def.getProducedClass().constructor.name} is marked as an Aggregate, but it doesn't have a static property called aggregateName`);
                }
                this.registerAggregateClass(name, def.getProducedClass());
            });
        }
        this.aggregateDefinitionsToRegister = null;
        if (this.commandDefinitionsToRegister && this.commandDefinitionsToRegister.length > 0) {
            this.commandDefinitionsToRegister.forEach((def) => {
                const name = def.getProducedClass().name;
                this.registerCommandClass(name, def.getProducedClass());
            });
        }
        this.commandDefinitionsToRegister = null;
    }
    newExecutionContext() {
        const execContext = new ExecutionContext_1.ExecutionContext(this.aggregateEventStore, this.commandExecutors, this.eventExecutors);
        execContext.setAggregateClasses(this.aggregateClasses);
        return execContext;
    }
    getEventStream() {
        return this.eventStream;
    }
    /**
     * Executes a single command and return the ExecutionContext
     * @param {Command[]} commands The command to execute
     * @returns {ExecutionContext} The execution context of the command
     */
    executeCommand(...commands) {
        return __awaiter(this, void 0, void 0, function* () {
            const executionContext = this.newExecutionContext();
            yield executionContext.executeCommand(...commands);
            return executionContext;
        });
    }
    getAggregateAs(aggregateId) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getAggregate(aggregateId));
        });
    }
    getAggregate(aggregateId) {
        return __awaiter(this, void 0, void 0, function* () {
            const cachedAggregate = this.aggregateCache.get(aggregateId);
            if (cachedAggregate) {
                return cachedAggregate;
            }
            const aggregate = yield this.getAggregateInternal(aggregateId);
            if (aggregate) {
                this.aggregateCache.set(aggregateId, aggregate);
            }
            return aggregate;
        });
    }
    getAggregateInternal(aggregateId) {
        return __awaiter(this, void 0, void 0, function* () {
            const allEvents = yield this.aggregateEventStore.getEventsOf(aggregateId);
            if (!allEvents || allEvents.length === 0) {
                return null;
            }
            return Aggregate_1.Aggregate.applyEvents(allEvents, this.eventExecutors, this.aggregateClasses);
        });
    }
    deserialiseCommand(obj, commandType) {
        if (!commandType && !Object.hasOwnProperty.call(obj, Command_1.Command.commandFieldType)) {
            throw new Error(`Can't deserialise object into typed instance because it doesn't have a ${Command_1.Command.commandFieldType} field`);
        }
        const type = commandType || obj[Command_1.Command.commandFieldType];
        if (!this.commandClasses.has(type)) {
            throw new Error(`Unknown command type ${type}`);
        }
        const typeConstructor = this.commandClasses.get(type);
        return Reflect.construct(typeConstructor, [obj]);
    }
    registerAggregateClass(name, aggregateClass) {
        Logger.info(`Registering aggregate ${name}`);
        this.aggregateClasses.set(name, aggregateClass);
    }
    registerCommandClass(name, commandClass) {
        Logger.info(`Registering command ${name}`);
        this.commandClasses.set(name, commandClass);
    }
    instantiateAggregate(aggregateType, aggregateId) {
        const aggregateClass = this.aggregateClasses.has(aggregateType) ? this.aggregateClasses.get(aggregateType) : Aggregate_1.Aggregate;
        return new aggregateClass(aggregateType, aggregateId);
    }
    registerCommandExecutor(commandExecutor) {
        this.commandExecutors.push(commandExecutor);
    }
    registerEventExecutor(eventExecutor) {
        this.eventExecutors.push(eventExecutor);
    }
}
__decorate([
    inceptum_1.AutowireGroup('cqrs:commandExecutor')
], CQRS.prototype, "commandExecutors", void 0);
__decorate([
    inceptum_1.AutowireGroup('cqrs:eventExecutor')
], CQRS.prototype, "eventExecutors", void 0);
__decorate([
    inceptum_1.AutowireGroupDefinitions('cqrs:aggregate')
], CQRS.prototype, "aggregateDefinitionsToRegister", void 0);
__decorate([
    inceptum_1.AutowireGroupDefinitions('cqrs:command')
], CQRS.prototype, "commandDefinitionsToRegister", void 0);
__decorate([
    inceptum_1.AutowireGroup('cqrs:eventListener')
], CQRS.prototype, "eventListeners", void 0);
__decorate([
    inceptum_1.StartMethod
], CQRS.prototype, "doSetup", null);
exports.CQRS = CQRS;
//# sourceMappingURL=CQRS.js.map