"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const stringify = require("json-stringify-safe");
const inceptum_1 = require("inceptum");
const Aggregate_1 = require("./Aggregate");
const AggregateEventStore_1 = require("./event/store/AggregateEventStore");
const AggregateCreatingCommand_1 = require("./command/AggregateCreatingCommand");
const CommandResult_1 = require("./command/CommandResult");
const ReturnToCallerError_1 = require("./error/ReturnToCallerError");
const EventExecutor_1 = require("./event/EventExecutor");
var Status;
(function (Status) {
    Status[Status["NOT_COMMMITED"] = 0] = "NOT_COMMMITED";
    Status[Status["COMMITTING"] = 1] = "COMMITTING";
    Status[Status["COMMITTED"] = 2] = "COMMITTED";
})(Status = exports.Status || (exports.Status = {}));
class ExecutionContext extends AggregateEventStore_1.AggregateEventStore {
    /**
     * Constructs a new instance of ExecutionContext
     * @param {AggregateEventStore} aggregateEventStore The store to commit events to
     */
    constructor(aggregateEventStore, commandExecutors, eventExecutors) {
        super();
        this.aggregateClasses = new Map();
        this.aggregateEventStore = aggregateEventStore;
        this.status = Status.NOT_COMMMITED;
        this.eventsToEmit = [];
        this.commandsToExecute = [];
        this.error = null;
        this.commandResults = new Map();
        this.commandExecutors = commandExecutors;
        this.eventExecutors = eventExecutors;
        // this.aggregateCache = new Map();
    }
    /**
     * Saves an aggregate event to this execution context.
     * The events won't be really saved on the aggregateEventStore until all
     * commands have been executed and the execution has been successful.
     * @param {aggregateEvent} aggregateEvent The aggregate event to store
     */
    commitEvent(event) {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateNotCommitted();
            this.eventsToEmit.push(event);
        });
    }
    /**
     * Adds a command to the queue of commands to be executed.
     * @param {AggregateCommand} aggregateCommand The command to add to the execution queue
     */
    addCommandToExecute(aggregateCommand) {
        this.validateNotCommitted();
        if (this.commandExecutors.findIndex((executor) => executor.canExecute(aggregateCommand)) < 0) {
            throw new Error(`Unknown command type: ${aggregateCommand.constructor.name}. There's no CommandExecutor registered for it`);
        }
        this.commandsToExecute.push(aggregateCommand);
    }
    /**
     * Validates that this execution context has not been committed yet.
     * @private
     */
    validateNotCommitted() {
        if (this.status >= Status.COMMITTED) {
            throw new Error('ExecutionContext is already committed. Can\'t perform additional actions');
        }
    }
    getAggregate(aggregateId) {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregateEvents = (yield this.aggregateEventStore.getEventsOf(aggregateId)) || [];
            const uncommittedEvents = this.getUncommittedEventsOf(aggregateId) || [];
            const allEvents = aggregateEvents.concat(uncommittedEvents);
            if (allEvents.length === 0) {
                return null;
            }
            return Aggregate_1.Aggregate.applyEvents(allEvents, this.eventExecutors, this.aggregateClasses);
        });
    }
    /**
     * Get all uncommitted events for the given aggregate id
     * @private
     * @param {string} aggregateId The id of the aggregate
     */
    getUncommittedEventsOf(aggregateId) {
        return this.eventsToEmit.filter((e) => EventExecutor_1.EventExecutor.getEventExecutor(e, this.eventExecutors).getAggregateId(e) === aggregateId);
    }
    /**
     * Executes a single command. This is a convenience method that calls both {@link addCommandToExecute} and
     * {@link commit}
     * @param {AggregateCommand} command The command to execute
     */
    executeCommand(...commands) {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateNotCommitted();
            commands.forEach((command) => {
                this.addCommandToExecute(command);
            });
            yield this.commit();
        });
    }
    /**
     * Commits the execution context.
     * This essentially will go through all commands pending execution and will call them in order. It will
     * fail on the first error that gets thrown by any of the commands, and this won't be able to be committed.
     */
    commit() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateNotCommitted();
            if (this.status === Status.COMMITTING) {
                throw new Error('ExecutionContext is already committing. Don\'t call commit directly, just call addCommandToExecute');
            }
            this.status = Status.COMMITTING;
            while (this.commandsToExecute.length > 0) {
                const command = this.commandsToExecute.shift();
                const commandExecutor = this.commandExecutors.find((executor) => executor.canExecute(command));
                const aggregate = (command instanceof AggregateCreatingCommand_1.AggregateCreatingCommand) ?
                    Aggregate_1.Aggregate.instantiateAggregate(command.getAggregateType(), command.getAggregateId(), this.aggregateClasses) :
                    yield this.getAggregate(command.getAggregateId());
                try {
                    yield commandExecutor.execute(command, this, aggregate);
                }
                catch (e) {
                    this.status = Status.COMMITTED;
                    if (e instanceof ReturnToCallerError_1.ReturnToCallerError) {
                        // return the error to the caller
                        this.error = e;
                    }
                    else {
                        // hide the actual error and return a custom error
                        this.error = new inceptum_1.ExtendedError(`There was an error executing command ${stringify(command)}`, e);
                    }
                    throw this.error;
                }
            }
            // All commands executed correctly
            this.status = Status.COMMITTED;
            try {
                yield this.aggregateEventStore.commitAllEvents(this.eventsToEmit);
            }
            catch (e) {
                if (e instanceof ReturnToCallerError_1.ReturnToCallerError) {
                    // return the error to the caller
                    this.error = e;
                }
                else {
                    // hide the actual error and return a custom error
                    this.error = new inceptum_1.ExtendedError('There was an error saving events', e);
                }
                throw this.error;
            }
        });
    }
    getError() {
        return this.error;
    }
    hasCommandResultForCommand(command) {
        return this.commandResults.has(command);
    }
    getCommandResultForCommand(command) {
        if (this.commandResults.has(command)) {
            return this.commandResults.get(command);
        }
        const result = new CommandResult_1.CommandResult(command);
        this.commandResults.set(command, result);
        return result;
    }
    getEventsOf(aggregateId) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.aggregateEventStore.getEventsOf(aggregateId);
        });
    }
    setAggregateClasses(aggregateClasses) {
        this.aggregateClasses = aggregateClasses;
    }
}
exports.ExecutionContext = ExecutionContext;
//# sourceMappingURL=ExecutionContext.js.map