import { ExecutionContext } from '../ExecutionContext';
import { Aggregate } from '../Aggregate';
import { Command } from './Command';
export declare abstract class CommandExecutor<T extends Command, A extends Aggregate> {
    /**
     * Validate the command previous to execution.
     *
     * If the command can't be executed, this method should throw an Error
     * that extends from {@link ValidationError}
     * @param command The command to validate
     * @param executionContext The execution context in which this command os to be executed
     * @param aggregate The (optional) aggregate this command acts on
     */
    abstract validate(command: T, executionContext: ExecutionContext, aggregate?: A): Promise<void>;
    /**
     * Executes the given command
     *
     * If the command throws an exception it will be transformed into a 500 error automatically by the {@link SwaggerCQRSMiddleware}.
     * If you want the error to be returned to the client, make sure your
     * error extends from {@link ReturnToCallerError} and specify any desired info.
     * @param command The command to execute
     * @param executionContext The execution context in which this command os to be executed
     * @param aggregate The (optional) aggregate this command acts on
     */
    abstract doExecute(command: T, executionContext: ExecutionContext, aggregate?: A): Promise<void>;
    /**
     * Validate the authorization needed to execute this command.
     *
     * If the command can't be executed, this method should throw an Error
     * that extends from {@link ../error/AuthorizationError}
     * @param command The command to validate authorization for
     * @param executionContext The execution context in which this command os to be executed
     * @param aggregate The (optional) aggregate this command acts on
     */
    abstract validateAuth(command: T, executionContext: ExecutionContext, aggregate?: A): Promise<void>;
    /**
     * Executed this command as part of the execution context.
     * @param {T} command
     * @param {ExecutionContext} executionContext
     * @param {A} aggregate? If the command is an instance of AggregateCommand an aggregate should be present
     * @returns {*}
     */
    execute(command: T, executionContext: ExecutionContext, aggregate?: A): Promise<void>;
    abstract canExecute(command: Command): boolean;
}
