"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const inceptum_1 = require("inceptum");
const stringify = require("json-stringify-safe");
const ReturnToCallerError_1 = require("../error/ReturnToCallerError");
const logger = inceptum_1.LogManager.getLogger(__filename);
const SWAGGER_CQRS_COMMAND_PROPERTY = 'x-inceptum-cqrs-command';
const SWAGGER_CQRS_GET_PROPERTY = 'x-inceptum-cqrs-get';
const newrelic = inceptum_1.NewrelicUtil.getNewrelicIfAvailable();
class SwaggerCQRSMiddleware {
    /**
     *
     * @param {CQRS} cqrs
     */
    constructor() {
        this.handlerCache = new Map();
    }
    setCQRS(cqrs) {
        this.cqrs = cqrs;
    }
    /**
     * @private
     * @param req
     * @returns {boolean}
     */
    hasCQRSCommand(req) {
        return !!this.getCQRSCommand(req);
    }
    hasCQRSGet(req) {
        return !!this.getCQRSGet(req);
    }
    /**
     * @private
     * @param req
     * @returns {*}
     */
    getCQRSCommand(req) {
        if (!req || !req.swagger) {
            return undefined;
        }
        if (req.swagger.operation && req.swagger.operation[SWAGGER_CQRS_COMMAND_PROPERTY]) {
            return req.swagger.operation[SWAGGER_CQRS_COMMAND_PROPERTY];
        }
        if (req.swagger.path && req.swagger.path[SWAGGER_CQRS_COMMAND_PROPERTY]) {
            return req.swagger.path[SWAGGER_CQRS_COMMAND_PROPERTY];
        }
        return undefined;
    }
    getCQRSGet(req) {
        if (!req || !req.swagger) {
            return undefined;
        }
        if (req.swagger.operation && req.swagger.operation[SWAGGER_CQRS_GET_PROPERTY]) {
            return req.swagger.operation[SWAGGER_CQRS_GET_PROPERTY];
        }
        if (req.swagger.path && req.swagger.path[SWAGGER_CQRS_GET_PROPERTY]) {
            return req.swagger.path[SWAGGER_CQRS_GET_PROPERTY];
        }
        return undefined;
    }
    register(expressApp) {
        const self = this;
        logger.debug('Registering CQRS Swagger middleware');
        expressApp.use((req, res, next) => __awaiter(this, void 0, void 0, function* () {
            try {
                if (self.hasCQRSCommand(req)) {
                    // There's a controller to be called.
                    yield self.handleCQRSCommand(req, res);
                }
                else if (self.hasCQRSGet(req)) {
                    yield self.handleCQRSGet(req, res);
                }
                else {
                    // No cqrs command defined, let it pass
                    next();
                }
            }
            catch (e) {
                next(e);
            }
        }));
    }
    /**
     * @private
     * @param req
     * @returns {Request}
     */
    handleCQRSCommand(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const cqrsCommand = this.getCQRSCommand(req);
            if (cqrsCommand.indexOf(':') > 0) {
                const parts = cqrsCommand.split(':', 2);
                return yield this.handleCQRSCommandWithName(req, res, parts[0], parts[1]);
            }
            return yield this.handleCQRSCommandWithName(req, res, cqrsCommand, 'body');
        });
    }
    /**
     * @private
     * @param req
     * @returns {Request}
     */
    handleCQRSGet(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const cqrsGet = this.getCQRSGet(req);
            if (cqrsGet.indexOf(':') > 0) {
                const parts = cqrsGet.split(':', 2);
                return yield this.handleCQRSGetWithName(req, res, parts[0], parts[1]);
            }
            return yield this.handleCQRSGetWithName(req, res, cqrsGet, 'id');
        });
    }
    /**
     * @private
     * @param req
     * @param res
     * @param commandName
     * @param paramName
     * @returns {*}
     */
    handleCQRSGetWithName(req, res, aggregateName, paramName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (newrelic && req.swagger && req.swagger.apiPath) {
                // NR adds a `/` at the start of the path for us.
                const basePath = req.swagger.swaggerObject.basePath.substring(1);
                newrelic.setTransactionName(`${basePath}${req.swagger.apiPath}`);
                newrelic.recordMetric(`Custom/CQRSGet/${aggregateName}`);
            }
            const id = this.getPayload(req, paramName);
            const aggregate = yield this.cqrs.getAggregate(id);
            if (!aggregate) {
                res.status(404);
                res.send(`Couldn't find aggregate id ${id}`);
            }
            else if (aggregateName !== aggregate.aggregateType) {
                res.status(400);
                res.send(`aggregate of aggregate id ${id} is NOT ${aggregateName}`);
            }
            else {
                res.send(aggregate);
            }
        });
    }
    /**
     * @private
     * @param req
     * @param res
     * @param commandName
     * @param bodyParamName
     * @returns {*}
     */
    handleCQRSCommandWithName(req, res, commandName, bodyParamName) {
        return __awaiter(this, void 0, void 0, function* () {
            res.header('Cache-Control', 'no-cache, no-store, must-revalidate, private');
            res.header('Expires', '-1');
            res.header('Pragma', 'no-cache');
            if (newrelic && req.swagger && req.swagger.apiPath) {
                // NR adds a `/` at the start of the path for us.
                const basePath = req.swagger.swaggerObject.basePath.substring(1);
                newrelic.setTransactionName(`${basePath}${req.swagger.apiPath}`);
                newrelic.recordMetric(`Custom/CQRSCommand/${commandName}`);
            }
            const payload = this.getPayload(req, bodyParamName);
            const command = this.cqrs.deserialiseCommand(payload || {}, commandName);
            try {
                const executionContext = yield this.cqrs.executeCommand(command);
                if (executionContext.hasCommandResultForCommand(command)) {
                    const commandResult = executionContext.getCommandResultForCommand(command);
                    if (commandResult.hasNewAggregateId()) {
                        res.header('Location', commandResult.getNewAggregateId());
                        res.status(201);
                    }
                    else {
                        res.status(200);
                    }
                    res.send(commandResult);
                }
                else {
                    res.status(204);
                    res.send('');
                }
            }
            catch (err) {
                logger.error(`Exception executing command ${commandName}: ${stringify(command)}`, err);
                if (newrelic) {
                    newrelic.noticeError(err);
                }
                if (err instanceof ReturnToCallerError_1.ReturnToCallerError) {
                    res.status(err.httpStatusCode);
                    res.send(err.getInfoToReturn());
                }
                else {
                    const internalError = 'Internal Server Error';
                    res.status(500);
                    res.send({ message: internalError });
                }
            }
        });
    }
    /**
     * @private
     * @param req
     * @param bodyParamName
     * @returns {*}
     */
    // tslint:disable-next-line:prefer-function-over-method
    getPayload(req, bodyParamName) {
        if (req.swagger.params && req.swagger.params[bodyParamName]) {
            return req.swagger.params[bodyParamName].value;
        }
        return undefined;
    }
}
exports.SwaggerCQRSMiddleware = SwaggerCQRSMiddleware;
//# sourceMappingURL=SwaggerCQRSMiddleware.js.map